/*
 * Copyright (c) 2005 The University of Notre Dame. All Rights Reserved.
 *
 * The University of Notre Dame grants you ("Licensee") a
 * non-exclusive, royalty free, license to use, modify and
 * redistribute this software in source and binary code form, provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. All express or implied conditions, representations and
 * warranties, including any implied warranty of merchantability,
 * fitness for a particular purpose or non-infringement, are hereby
 * excluded.  The University of Notre Dame and its licensors shall not
 * be liable for any damages suffered by licensee as a result of
 * using, modifying or distributing the software or its
 * derivatives. In no event will the University of Notre Dame or its
 * licensors be liable for any lost revenue, profit or data, or for
 * direct, indirect, special, consequential, incidental or punitive
 * damages, however caused and regardless of the theory of liability,
 * arising out of the use of or inability to use software, even if the
 * University of Notre Dame has been advised of the possibility of
 * such damages.
 *
 * SUPPORT OPEN SCIENCE!  If you use OpenMD or its source code in your
 * research, please cite the appropriate papers when you publish your
 * work.  Good starting points are:
 *                                                                      
 * [1]  Meineke, et al., J. Comp. Chem. 26, 252-271 (2005).             
 * [2]  Fennell & Gezelter, J. Chem. Phys. 124, 234104 (2006).          
 * [3]  Sun, Lin & Gezelter, J. Chem. Phys. 128, 234107 (2008).          
 * [4]  Kuang & Gezelter,  J. Chem. Phys. 133, 164101 (2010).
 * [5]  Vardeman, Stocker & Gezelter, J. Chem. Theory Comput. 7, 834 (2011).
 */
 
#ifndef CONSTRAINTS_SHAKE_HPP
#define CONSTRAINTS_SHAKE_HPP

#include "brains/SimInfo.hpp"
#include "constraints/ConstraintPair.hpp"
#include "io/ConstraintWriter.hpp"

namespace OpenMD {

  class Shake {
  public:
    enum ConsStatus{
      consFail = -1,   //Constraint Fail
      consSuccess = 0, //constrain the pair by moving two elements
      consAlready = 1  //current pair is already constrained, do not need to move the elements
    }; 
    
    Shake(SimInfo* info);
    void constraintR();
    void constraintF();
    
    int getMaxConsIteration() { return maxConsIteration_; }
    void setMaxConsIteration(int iteration) { maxConsIteration_ = iteration; }
    
    RealType getConsTolerance() { return consTolerance_; } 
    void setConsTolerance(RealType tolerance) { consTolerance_ = tolerance;}           
  private:
    typedef int (Shake::*ConstraintPairFuncPtr)(ConstraintPair*);
    void doConstraint(ConstraintPairFuncPtr func);
    int constraintPairR(ConstraintPair* consPair);
    int constraintPairF(ConstraintPair* consPair);
    
    SimInfo* info_;
    int maxConsIteration_;        
    RealType consTolerance_;
    Snapshot* currentSnapshot_;   
    bool doShake_;
    std::string constraintOutputFile_;
    ConstraintWriter* constraintWriter_;
    RealType constraintTime_;
    RealType currConstraintTime_;
  };
}
#endif
